/*
 * say_fr.c
 */

/*! \brief  say_number_full_fr: French syntax */
/* 	Extra sounds needed:
 	1F: feminin 'une'
 	et: 'and' */
static int say_number_full_fr(const say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	int playa = 0;
	int mf = 1;                            /* +1 = male; -1 = female */
	char fn[256] = "";
	if (!num) 
		return say_digits_full(a, 0);
	
	if (options && !strncasecmp(options, "f",1))
		mf = -1;

	while(!res && (num || playh || playa)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else if (playa) {
			snprintf(fn, sizeof(fn), "digits/et");
			playa = 0;
		} else if (num == 1) {
			if (mf < 0)
				snprintf(fn, sizeof(fn), "digits/%dF", num);
			else
				snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 21) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 70) {
			snprintf(fn, sizeof(fn), "digits/%d", (num/10)*10);
			if ((num % 10) == 1) playa++;
			num = num % 10;
		} else if (num < 80) {
			snprintf(fn, sizeof(fn), "digits/60");
			if ((num % 10) == 1) playa++;
			num = num - 60;
		} else if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/80");
			num = num - 80;
		} else if (num < 200) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			num = num - 100;
		} else if (num < ONE_THOUSAND) {
			snprintf(fn, sizeof(fn), "digits/%d", (num/100));
			playh++;
			num = num % 100;
		} else if (num < 2000) {
			snprintf(fn, sizeof(fn), "digits/thousand");
			num = num - ONE_THOUSAND;
		} else if (num < ONE_MILLION) {
			res = say_number_full_fr(a, num / ONE_THOUSAND, options);
			if (res)
				break;
			snprintf(fn, sizeof(fn), "digits/thousand");
			num = num % ONE_THOUSAND;
		} else	if (num < ONE_BILLION) {
			res = say_number_full_fr(a, num / ONE_MILLION, options);
			if (res)
				break;
			snprintf(fn, sizeof(fn), "digits/million");
			num = num % ONE_MILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* French syntax 
oclock = heure
*/
static int say_date_with_format_fr(const say_args_t *a, const struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c;
	say_date_with_format_f *f = say_date_with_format_fr; /* shorthand for recursive call */

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case 'm':
			/* First - Twelfth */
			res = s_streamwait(a, "digits/h-%d", tm.tm_mon +1);
			break;
		case 'd':
		case 'e':
			/* First */
			if (tm.tm_mday == 1)
				res = s_streamwait3(a, "digits/h-1");
			else
				res = say_number_full_fr(a, tm.tm_mday, NULL);
			break;
		case 'Y':
			/* Year */
			if (tm.tm_year > 99) {
				res = s_streamwait3(a,"digits/2");
				if (!res)
					res = s_streamwait3(a, "digits/thousand");
				if (tm.tm_year > 100) {
					if (!res)
						res = say_number_full_fr(a, tm.tm_year - 100, NULL);
				}
			} else {
				if (tm.tm_year < 1) {
					/* I'm not going to handle 1900 and prior */
					/* We'll just be silent on the year, instead of bombing out. */
				} else {
					res = s_streamwait3(a, "digits/thousand");
					if (!res) {
						s_streamwait3(a, "digits/9");
						s_streamwait3(a, "digits/hundred");
						res = say_number_full_fr(a, tm.tm_year, NULL);
					}
				}
			}
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			res = s_streamwait(a, "digits/%d", hr_to_12( tm.tm_hour));
			if (!res)
				res = s_streamwait3(a, "digits/oclock");
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			res = say_number_full_fr(a, tm.tm_hour, NULL);
			if (!res) {
				if (format[offset] == 'H')
					res = s_streamwait3(a, "digits/oclock");
			}
			break;
		case 'M': /* Minute */
			if (tm.tm_min > 0)
				res = say_number_full_fr(a, tm.tm_min, NULL);
			break;
		case 'S': /* Seconds */
			res = say_number_full_fr(a, tm.tm_hour, NULL);
			if (!res)
				res = s_streamwait3(a, "digits/second");
			break;
		case 'x':
			res = f(a, &tm, "AdBY", delta);
			break;

		default:
			res = say_date_common(a, &tm, format, delta, &offset, f);
		}
	}
	return res;
}

/* French syntax */
static int say_datetime_from_now_fr(const say_args_t *a, time_t t)
{
	int res=0;
	time_t nowt;
	int daydiff;
	struct tm tm;
	struct tm now;
	char fn[256];

	time(&nowt);

	localtime_r(&t,&tm);
	localtime_r(&nowt,&now);
	daydiff = now.tm_yday - tm.tm_yday;
	if ((daydiff < 0) || (daydiff > 6)) {
		/* Day of month and month */
		if (!res) {
			snprintf(fn, sizeof(fn), "digits/mon-%d", tm.tm_mon);
			res = ast_streamfile(a->chan, fn, a->language);
			if (!res)
				res = ast_waitstream(a->chan, a->ints);
		}
		if (!res)
			res = say_number_full_fr(a, tm.tm_mday, NULL);

	} else if (daydiff) {
		/* Just what day of the week */
		if (!res)
			res = s_streamwait(a, "digits/day-%d", tm.tm_wday);
	} /* Otherwise, it was today */
	if (!res)
		res = ast_say_time(a->chan, t, a->ints, a->language);
	return res;
}

static struct _lang_sw map_fr = {
	.language = "fr",
	.say_number_full = say_number_full_fr,
	.say_date_with_format = say_date_with_format_fr,
	.say_datetime_from_now = say_datetime_from_now_fr,
	.date_fmt = "AdBY",
	.time_fmt = "HM",
	.datetime_fmt = "dABHMY", /* XXX */
};

