/*
 * say_nl.c
 */

/*! \brief  say_number_full_nl: dutch syntax */
/* New files: digits/nl-en
 */
static int say_number_full_nl(const say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	int units = 0;
	char fn[256] = "";
	if (!num) 
		return say_digits_full(a, 0);
	while (!res && (num || playh )) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			units = num % 10;
			if (units > 0) {
				res = say_number_full_nl(a, units, options);
				if (res)
					return res;
				num = num - units;
				snprintf(fn, sizeof(fn), "digits/nl-en");
			} else {
				snprintf(fn, sizeof(fn), "digits/%d", num - units);
				num = 0;
			}
		} else {
			if (num < ONE_THOUSAND) {
				snprintf(fn, sizeof(fn), "digits/%d", (num/100));
				playh++;
				num -= ((num / 100) * 100);
			} else {
				if (num < ONE_MILLION) { /* 1,000,000 */
					res = say_number_full_nl(a, num / ONE_THOUSAND, options);
					if (res)
						return res;
					num = num % ONE_THOUSAND;
					snprintf(fn, sizeof(fn), "digits/thousand");
				} else {
					if (num < ONE_BILLION) { /* 1,000,000,000 */
						res = say_number_full_nl(a, num / ONE_MILLION, options);
						if (res)
							return res;
						num = num % ONE_MILLION;
						snprintf(fn, sizeof(fn), "digits/million");
					} else {
						ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
						res = -1;
					}
				}
			}
		}

		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* Dutch syntax */
static int say_date_with_format_nl(const say_args_t *a, const struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c;
	say_date_with_format_f *f = say_date_with_format_nl; /* shorthand for recursive call */

	for (offset=0 ; res ==  0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case 'm':
			/* First - Twelfth */
			res = s_streamwait(a, "digits/h-%d", tm.tm_mon +1);
			break;
		case 'd':
		case 'e':
			/* First - Thirtyfirst */ /* XXX enum perhaps ? */
			res = say_number_full_nl(a, tm.tm_mday, NULL);
			break;
		case 'Y':
			/* Year */
			if (tm.tm_year < 1) {
				/* I'm not going to handle 1900 and prior */
				/* We'll just be silent on the year, instead of bombing out. */
			} else if (tm.tm_year <= 99) {
				res = s_streamwait3(a, "digits/19");
				if (!res && tm.tm_year <= 9) /* 1901 - 1909 */
					res = s_streamwait3(a, "digits/oh");
				if (!res)
					res = say_number_full_nl(a, tm.tm_year, NULL);
			} else
				res = say_number_full_nl(a, tm.tm_year + 1900, NULL);
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			res = s_streamwait(a, "digits/%d", hr_to_12(tm.tm_hour));
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			res = say_number_full_nl(a, tm.tm_hour, NULL);
			if (!res)
				res = s_streamwait3(a, "digits/nl-uur");
			break;
		case 'M':
			/* Minute */
			res = say_number_full_nl(a, tm.tm_min, NULL);
			break;
		case 'S':
			/* Seconds */
			if (tm.tm_sec > 0 && tm.tm_sec < 10)
				res = s_streamwait3(a, "digits/oh");
			if (!res)
				res = say_number_full_nl(a, tm.tm_min, NULL);
			break;
		case 'x':
			res = f(a, &tm, "ABdY", delta);
			break;
		default:
			res = say_date_common(a, &tm, format, delta, &offset, f);
		}
	}
	return res;
}

static struct _lang_sw map_nl = {
	.language = "nl",
	.say_number_full = say_number_full_nl,
	.say_date_with_format = say_date_with_format_nl,
	.date_fmt = "AdBY",
	.time_fmt = "HM",
	.datetime_fmt = "AdBYHM",
};

